import { getHeaders, type H3Event } from 'h3'
import { useCDNHeaders } from '#nuxt-multi-cache/composables'
import { defineGraphqlServerOptions } from 'nuxt-graphql-middleware/dist/runtime/serverOptions'
import { extractCacheability } from '~/server/utils/cacheability'
import type { GraphqlCacheability } from '~/server/helpers'
import { MAX_AGE } from '~/server/helpers'

const HEADER_KEYS: string[] = [
  'x-forwarded-for',
  'x-forwarded-host',
  'x-forwarded-server',
  'x-forwarded-proto',
  'x-forwarded-port',
  'x-client-ip',
  'x-real-ip',
  'x-client-ssl',
  'sec-fetch-site',
  'sec-ch-ua-platform',
  'sec-ch-ua',
  'referer',
  'user-agent',
  'user-agent-https',
  'cookie',
  'authorization',
]

export default defineGraphqlServerOptions<{
  __cacheability?: GraphqlCacheability
}>({
  graphqlEndpoint(event: H3Event | undefined) {
    const config = useRuntimeConfig()
    if (event) {
      const query = getQuery(event)
      const config = useRuntimeConfig()
      const candidates = query.__l || query.language || query.langcode
      let language = Array.isArray(candidates) ? candidates[0] : candidates

      // Fix for the frontpage, where no language is available.
      if (!language) {
        language = config.public.languageNegotiation.availableLanguages[0]
      }

      if (
        language &&
        typeof language === 'string' &&
        config.public.languageNegotiation.availableLanguages.includes(language)
      ) {
        return `${config.backendUrl}/${language}/graphql`
      }
    }

    // Used when fetching schema.
    return `${config.backendUrl}/de/graphql`
  },
  serverFetchOptions(event: H3Event | undefined) {
    if (event) {
      const config = useRuntimeConfig()
      const incomingHeaders = getHeaders(event) as Record<string, string>

      const headers: Record<string, string> = {
        'x-drupal-graphql-token': config.drupalGraphqlToken,
      }

      HEADER_KEYS.forEach((key) => {
        const value = incomingHeaders[key]
        if (value) {
          headers[key] = value
        }
      })

      return {
        headers,
      }
    }

    return {}
  },
  onServerResponse(event: H3Event, graphqlResponse, operation, operationName) {
    // Pass the set-cookie header from the GraphQL response to the client.
    const setCookie = graphqlResponse.headers.get('set-cookie')

    if (setCookie) {
      event.node.res.setHeader('set-cookie', setCookie)
    }

    const cacheability = extractCacheability(graphqlResponse, event)

    const hasMessages = !!(
      graphqlResponse._data?.data &&
      'messengerMessages' in graphqlResponse._data.data &&
      graphqlResponse._data.data.messengerMessages?.length
    )

    // Mark as uncacheable if the response contains Drupal messages.
    if (hasMessages) {
      cacheability.isCacheable = false
    }

    // This will set the correct headers for the CDN for all GraphQL requests.
    useCDNHeaders((helper) => {
      if (operation === 'query' && operationName && cacheability.isCacheable) {
        return helper
          .public()
          .setNumeric('maxAge', cacheability.maxAge)
          .set('staleIfError', MAX_AGE.ONE_DAY)
          .set('staleWhileRevalidate', MAX_AGE.TWO_HOURS)
          .set('mustRevalidate', true)
          .addTags(cacheability.tagsCdn)
          .addTags(['nuxt:api:graphql'])
      }

      return helper.private()
    }, event)

    // This is only provided when the request originates from SSR.
    // We pass information about the cacheability along the GraphQL response,
    // so that the Nuxt app is able to use this to determine if the
    // rendered page can be cached or not.
    const addCacheability =
      getQuery(event).__server === 'true' || import.meta.dev

    // Return the GraphQL response.
    return {
      data: graphqlResponse._data!.data,
      errors: graphqlResponse._data!.errors,
      __cacheability: addCacheability ? cacheability : undefined,
    }
  },
  onServerError(event) {
    // Directly set the response status so we don't render the Nuxt 404 page.
    setResponseStatus(event, 500)
    useCDNHeaders((v) => v.private(), event)
  },
})
