import { resolve } from 'node:path'
import resolveConfig from 'tailwindcss/resolveConfig'
import { runtimeConfig } from './config/runtimeConfig'
import graphqlMiddleware from './config/graphqlMiddleware'
import multiCache from './config/multiCache'
import tailwindConfig from './tailwind.config'

const fullTailwindConfig = resolveConfig(tailwindConfig)
const ONE_YEAR = 31_536_000
const IS_DEV = process.env.NODE_ENV === 'development'
const NUXT_REQUEST_HOST = process.env.NUXT_REQUEST_HOST

const LANGCODES = ['de', 'fr', 'en']

// https://nuxt.com/docs/api/configuration/nuxt-config
export default defineNuxtConfig({
  devtools: { enabled: true },
  telemetry: false,
  graphqlMiddleware,
  multiCache,
  runtimeConfig,

  alias: {
    '#nuxt-svg-sprite': resolve(__dirname, './.nuxt/nuxt-svg-sprite'),
    'tailwind-config': resolve(__dirname, './tailwind.config.ts'),
    '#vuepal/rokka': resolve(__dirname, './.nuxt/vuepal-rokka.ts'),
  },

  app: {
    rootId: 'nuxt-root',
    pageTransition: { name: 'page', mode: 'out-in' },
    head: {
      title: 'Starterkit',
      htmlAttrs: {
        tagDuplicateStrategy: 'merge',
        lang: 'de',
      },
    },
  },

  /**
   * Nuxt.js modules
   */
  modules: [
    '@nuxt/test-utils/module',
    '@formkit/nuxt',
    '@nuxtjs/tailwindcss',
    'nuxt-graphql-middleware',
    'nuxt-language-negotiation',
    'nuxt-easy-texts',
    '@rokka-io/nuxt',
    'nuxt-page-dependencies',
    'nuxt-multi-cache',
    'nuxt-plugin-order',
    'vuepal',
    'nuxt-svg-icon-sprite',
    '@blokkli/editor',
    'nuxt-viewport',
    '@nuxt/eslint',
  ],

  easyTexts: {
    pattern: [
      './components/**/*.{js,ts,vue}',
      './pages/**/*.{js,ts,vue}',
      './composables/**/*.{js,ts,vue}',
      './*.{js,ts,vue}',
    ],
    generators: [
      {
        outputPath: './queries/translations.graphql',
        generate: 'drupal-graphql-texts',
      },
    ],
  },

  rokka: {
    host: '',
    viewports: {
      xs: 640,
      sm: 768,
      md: 1024,
      lg: 1280,
      xl: 1380,
    },
    dpr: ['', '1.5', '2'],
  },

  pageDependencies: {
    checkComposableCalled: true,
  },

  pluginOrder: {
    order: [
      // Match plugins by their file path.
      // Note that only a single plugin may match.
      // The match is performed as "plugin.src.includes(v.pathMatch)".
      { pathMatch: 'plugins/language.ts' },
      { pathMatch: '@blokkli/editor/dist/runtime/plugins/blokkliEditable.js' },
      { pathMatch: 'staticNodes.mjs' },
      { pathMatch: 'formkitSSRPlugin.mjs' },
      { pathMatch: 'clickaway.ts' },
      { pathMatch: 'graphqlMiddleware.ts' },
      { pathMatch: 'forceHttps.js' },
      { pathMatch: 'nuxt-easy-texts/dist/runtime/plugins/texts.mjs' },
      { pathMatch: 'drupalUser.ts' },
    ],

    logSortedPlugins: false,
  },

  vuepal: {
    frontendRouting: {
      enabled: true,
      langcodes: LANGCODES,
      outputPath: './../drupal/config/default/frontend_routing.settings.yml',
    },
    adminToolbar: {
      enabled: true,
    },
    localTasks: {
      enabled: true,
    },
    drupalRoute: {
      enabled: true,
    },
    devMode: {
      enabled: true,
      url: `https://${NUXT_REQUEST_HOST}`,
      forceHttps: true,
    },
  },

  sourcemap: true,

  nitro: {
    compressPublicAssets: true,
  },

  hooks: {
    'vite:extend'({ config }) {
      if (config.server && config.server.hmr) {
        // @ts-ignore
        config.server.hmr.protocol = 'wss'
      }
    },
  },

  vite: {
    css: {
      devSourcemap: true,
    },

    plugins: [],
  },

  formkit: {
    // Experimental support for auto loading (see note):
    autoImport: true,
  },

  languageNegotiation: {
    // Define the available languages.
    availableLanguages: LANGCODES,
    defaultLanguageNoPrefix: false,

    // We use two negotiators: Path prefix takes precedence. In cases where no
    // path prefix is available, we fall back to Accept-Language headers.
    negotiators: ['pathPrefix', 'acceptLanguage'],
  },

  svgIconSprite: {
    sprites: {
      default: {
        importPatterns: ['./assets/symbols/**/*.svg'],
      },
    },
  },

  postcss: {
    plugins: {
      'postcss-import': {},
      'tailwindcss/nesting': {},
      tailwindcss: {},
      'postcss-hexrgba': {
        colorFunctionNotation: 'modern',
        transformToBareValue: true,
      },
      cssnano: {
        preset: 'default',
      },
    },
  },

  routeRules: {
    '/_nuxt/**': {
      cache: {
        maxAge: ONE_YEAR,
      },
      headers: {
        'cache-control': `public,max-age=${ONE_YEAR},s-maxage=${ONE_YEAR}`,
      },
    },
    '/fonts/**': {
      cache: {
        maxAge: ONE_YEAR,
      },
      headers: {
        'cache-control': `public,max-age=${ONE_YEAR},s-maxage=${ONE_YEAR}`,
      },
    },
  },

  blokkli: {
    pattern: ['components/Paragraph/**/*.{js,ts,vue}'],
    globalOptions: {
      teaserStyle: {
        type: 'radios',
        label: 'Display',
        default: 'grid',
        options: {
          grid: 'Grid',
          staggered_grid: 'Staggered Grid',
        },
      },
      mobileStyle: {
        type: 'radios',
        label: 'Mobile display',
        default: 'stapel',
        options: {
          stack: 'Stack',
          slider: 'Slider',
        },
      },
      spacing: {
        type: 'radios',
        label: 'Spacing',
        default: 'none',
        displayAs: 'icons',
        options: {
          none: { label: 'no spacing', icon: 'icon-blokkli-option-no-spacing' },
          small: {
            label: 'small spacing',
            icon: 'icon-blokkli-option-small-spacing',
          },
          large: {
            label: 'large spacing',
            icon: 'icon-blokkli-option-large-spacing',
          },
        },
      },
      imageFormat: {
        type: 'radios',
        label: 'Image Format',
        default: 'full',
        displayAs: 'icons',
        options: {
          full: { label: 'extra large', icon: 'icon-blokkli-option-xlarge' },
          big: { label: 'large', icon: 'icon-blokkli-option-large' },
          text: { label: 'medium', icon: 'icon-blokkli-option-medium' },
          small: { label: 'small', icon: 'icon-blokkli-option-small' },
        },
      },
    },
    translations: {
      en: {
        editIndicatorLabel: 'Edit paragraphs',
      },
      de: {
        editIndicatorLabel: 'Abschnitte bearbeiten',
      },
    },
    chunkNames: ['global', 'rare'],
    itemEntityType: 'paragraph',
    storageDefaults: {
      blockFavorites: ['text'],
    },
    schemaOptionsPath: IS_DEV
      ? '../drupal/docroot/modules/custom/blokkli_starterkit/data/schema.json'
      : undefined,
    defaultLanguage: 'de',
    // Make sure the editor is always rendered in German instead of the
    // current page language.
    forceDefaultLanguage: true,
  },

  viewport: {
    breakpoints: {
      ...Object.fromEntries(
        Object.entries(fullTailwindConfig.theme.screens).map((entry) => {
          return [entry[0], parseInt(entry[1])]
        }),
      ),
    },
  },

  experimental: {
    asyncContext: true,
  },

  compatibilityDate: '2024-08-14',
})
